;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;							;;
;;	Semantic Constraints	;; 
;;							;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(deftemplate IS (multislot CASE) (multislot GENDER) (multislot ARTICLE) (multislot ANSWER))
(deftemplate SS (multislot CASE) (multislot GENDER) (multislot ARTICLE) (multislot ANSWER))

; -----------------------------------------------------------------------------------------------
; A couple of constants that let us set a variable value in the LHS (TODO - find a better way!!!)
; -----------------------------------------------------------------------------------------------
(defrule initialise
=>
	(assert (satisfied SATISFIED))
	(assert (violated VIOLATED))
	)

;; NOTE: The answer must be passed in a letter at a time!

;; --------------------------------------
;;                 CASE
;; --------------------------------------

(defrule sem-case-N 
	        (IS (CASE N))
	        (SS (CASE ?case))
          =>
	        (bind ?fb (fetch FEEDBACK))
			(if (eq ?case N) 
			then
				 (?fb addSatisfiedConstraint "sem-case-N")
			else
			     (?fb addMessage "Oops - wrong case. The nominative case is used for the subject of a sentence, the noun that carries out the verb.")
			 	 (?fb addViolatedConstraint "sem-case-N")
			)	 	 	
)

(defrule sem-case-A 
         	(IS (CASE A))
         	(SS (CASE ?case))
          =>
         	(bind ?fb (fetch FEEDBACK))
		 	(if (eq ?case A) 
		 	then
			 	(?fb addSatisfiedConstraint "sem-case-A")
		 	else
		        (?fb addMessage "Oops - wrong case. The accusative case is used for the object of a sentence, and after some prepositions.")
		 	 	(?fb addViolatedConstraint "sem-case-A")
		 	)	 	 	
)

(defrule semantic-case-D 
         (IS (CASE D))
         (SS (CASE ?case))
    =>
         (bind ?fb (fetch FEEDBACK))
		 (if (eq ?case D) 
		 then
			(?fb addSatisfiedConstraint "sem-case-D")
		 else
		 	(?fb addMessage "Oops - wrong case. The dative case is used for the indirect object, and after some prepositions.")
		 	(?fb addViolatedConstraint "sem-case-D")
		 )	 	 	
)
        
(defrule sem-case
          (IS (CASE ? $?)) 

		  (or
	         (and (SS (CASE ? $?)) 				(satisfied ?sat))
	         (and (not (SS (CASE ? $?)))	 	(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-case")
		 else	
	     	(?fb addMessage "You haven't specified a case")
	   	 	(?fb addViolatedConstraint "sem-case")
	   	 )
)
                                       
;; --------------------------------------
;;                GENDER
;; --------------------------------------
        
(defrule sem-gender-M
         (IS (GENDER M))
         (SS (GENDER ?gender))
    =>
         (bind ?fb (fetch FEEDBACK))
		 (if (eq ?gender M)
		 then
			 (?fb addSatisfiedConstraint "sem-gender-M")
		 else
		     (?fb addMessage "Oops - wrong gender. The gender for each noun must be learnt individually, but the article can sometimes provide a clue.")
		 	 (?fb addViolatedConstraint "sem-gender-M")
		 )	 	 	
)

(defrule sem-gender-F
         (IS (GENDER F))
         (SS (GENDER ?gender))
    =>
         (bind ?fb (fetch FEEDBACK))
		 (if (eq ?gender F) 
		 then
			 (?fb addSatisfiedConstraint "sem-gender-F")
		 else
		     (?fb addMessage "Oops - wrong gender. The gender for each noun must be learnt individually, but the article can sometimes provide a clue.")
		 	 (?fb addViolatedConstraint "sem-gender-F")
		 )	 	 	
)
        
(defrule sem-gender-N
         (IS (GENDER N))
         (SS (GENDER ?gender))
    =>
         (bind ?fb (fetch FEEDBACK))
		 (if (eq ?gender N) 
		 then
			 (?fb addSatisfiedConstraint "sem-gender-N")
		 else
		     (?fb addMessage "Oops - wrong gender. The gender for each noun must be learnt individually, but the article can sometimes provide a clue.")
		 	 (?fb addViolatedConstraint "sem-gender-N")
		 )	 	 	
)
        
(defrule sem-gender-P
         (IS (GENDER P))
         (SS (GENDER ?gender))
    =>
         (bind ?fb (fetch FEEDBACK))
		 (if (eq ?gender P) 
		 then
			 (?fb addSatisfiedConstraint "sem-gender-P")
		 else
		     (?fb addMessage "Oops - wrong gender. The gender for each noun must be learnt individually, but the article can sometimes provide a clue.")
		 	 (?fb addViolatedConstraint "sem-gender-P")
		 )	 	 	
)
        
(defrule sem-gender
          (IS (GENDER ? $?)) 

		  (or
	         (and (SS (GENDER ? $?)) 				(satisfied ?sat))
	         (and (not (SS (GENDER ? $?)))		 	(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-gender")
		 else	
	     	(?fb addMessage "You have not specified a gender")
	   	 	(?fb addViolatedConstraint "sem-gender")
	   	 )
)

;; --------------------------------------
;;                ARTICLE
;; --------------------------------------
        
(defrule sem-article-I
         (IS (ARTICLE I))
         (SS (ARTICLE ?article))
    =>
         (bind ?fb (fetch FEEDBACK))
		 (if (eq ?article I) 
		 then
			 (?fb addSatisfiedConstraint "sem-article-I")
		 else
		     (?fb addMessage "Oops - wrong article. Ein-words are ein/eine/etc, but also kein, mein, dein, sein, ihr, unser, euer, ihr and Ihr.")
		 	 (?fb addViolatedConstraint "sem-article-I")
		 )	 	 	
)

(defrule sem-article-D
         (IS (ARTICLE D))
         (SS (ARTICLE ?article))
    =>
         (bind ?fb (fetch FEEDBACK))
		 (if (eq ?article D) 
		 then
			 (?fb addSatisfiedConstraint "sem-article-D")
		 else
		     (?fb addMessage "Oops - wrong article. Der-words are der/die/das/etc, but also dieser, jeder, solcher, welcher, alle, and jener.")
		 	 (?fb addViolatedConstraint "sem-article-D")
		 )	 	 	
)
        
(defrule sem-article-N
         (IS (ARTICLE N))
         (SS (ARTICLE ?article))
    =>
         (bind ?fb (fetch FEEDBACK))
		 (if (eq ?article N) 
		 then
			 (?fb addSatisfiedConstraint "sem-article-N")
		 else
		     (?fb addMessage "Oops - wrong article. There is no article when the adjective stands alone before the noun.  There is an article here.")
		 	 (?fb addViolatedConstraint "sem-article-N")
		 )	 	 	
)
        
(defrule sem-article
          (IS (ARTICLE ? $?)) 

		  (or
	         (and (SS (ARTICLE ? $?)) 				(satisfied ?sat))
	         (and (not (SS (ARTICLE ? $?)))		 	(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-article")
		 else	
	     	(?fb addMessage "You haven't specified an article")
	   	 	(?fb addViolatedConstraint "sem-article")
	   	 )
)
        
;; ***********************************
;; These are common to both interfaces
;; ***********************************

; ------------------
; Spelled correctly?        
; ------------------
(defrule sem-spelling-1
         (IS (ANSWER $?w1 ? ?))
         (SS (ANSWER ? $?))

		 (or 
         	(and (SS (ANSWER $?w1 $?)) 			(satisfied ?sat))
         	(and (not (SS (ANSWER $?w1 $?)))	(violated ?sat))
         )
   =>
         (bind ?fb (fetch FEEDBACK))
		 (if (eq ?sat SATISFIED)
		 then
			 (?fb addSatisfiedConstraint "sem-spelling-1")
	     else
	     	 (?fb addMessage "You have spelled the adjective incorrectly.")
	   	 	 (?fb addViolatedConstraint "sem-spelling-1")
		)
)


(defrule sem-spelling-2
         (IS (ANSWER $?w1 ?l1 ?l2))
         (SS (ANSWER $?w1 $?))
         (SS (ANSWER $? ?l2))

		 (or
	         (and (SS (ANSWER $?w1 ?l1 ?l2)) 		(satisfied ?sat))
	         (and (not (SS (ANSWER $?w1 ?l1 ?l2)))	(violated  ?sat))
	     )

    =>
         (bind ?fb (fetch FEEDBACK))
		 (if (eq ?sat SATISFIED)
		 then
		   	 (?fb addSatisfiedConstraint "sem-spelling-2")
		 else
		     (?fb addMessage "You have unnecessary letters in your answer.")
		   	 (?fb addViolatedConstraint "sem-spelling-2")
		)
)

; ------------------------------------
;              ENDINGS
; ------------------------------------

(defrule sem-1
         (SS (ARTICLE D))
         (SS (ANSWER ? $?))

		 (or 
			(and 
				(or (SS (ANSWER $? e n)) (SS (ANSWER $? e)))
				(satisfied ?sat)
			)
		 	         
	        (and
	        	(not (or (SS (ANSWER $? e n)) (SS (ANSWER $? e))))
				(violated ?sat)	        	
         	)
         )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
          then 
			 (?fb addSatisfiedConstraint "sem-1")
		  else
	     	 (?fb addMessage "If an adjective is preceded by a 'der-word', it must end in either -e or -en.")
	   	 	 (?fb addViolatedConstraint "sem-1")
	   	 )
)

(defrule sem-2
          (or         
           	(and
            	(SS (ARTICLE D))
            	(or (SS (ANSWER $? e n)) (SS (ANSWER $? e)))
            )
           	(and
            	(SS (ARTICLE I))
            	(or (SS (ANSWER $? e)) (SS (ANSWER $? e n)) (SS (ANSWER $? e r)) (SS (ANSWER $? e s)))
            )
          )
         (SS (GENDER P))

		 (or         
          (and (SS (ANSWER $? e n))  		(satisfied ?sat))
          (and (not (SS (ANSWER $? e n)))  	(violated  ?sat))
         )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
          then 
		   	 (?fb addSatisfiedConstraint "sem-2")
		  else
	     	 (?fb addMessage "If an adjective is plural, and preceded by a der or ein word, it must take the ending -en.")
	   	 	 (?fb addViolatedConstraint "sem-2")
		  )
)
        
(defrule sem-3
         (SS (ARTICLE D))
         (not (SS (GENDER P)))
         (SS (CASE N))
         (or (SS (ANSWER $? e n)) (SS (ANSWER $? e)))
         
         (or
         	(and (SS (ANSWER $? e))        		(satisfied ?sat))
         	(and (not (SS (ANSWER $? e))) 		(violated  ?sat))
         )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
          then 
		 	(?fb addSatisfiedConstraint "sem-3")
		  else
		     (?fb addMessage "If a noun is nominative and singular, and preceded by a 'der-word', it must end in -e, no matter what the gender.")
		   	 (?fb addViolatedConstraint "sem-3")
		  )
)
                
(defrule sem-4
          (or
           	(and
            	(or (SS (ANSWER $? e n)) (SS (ANSWER $? e)))
            	(SS (ARTICLE D))
            )
           	(and
            	(or (SS (ANSWER $? e)) (SS (ANSWER $? e n)) (SS (ANSWER $? e r)) (SS (ANSWER $? e s)))
            	(SS (ARTICLE I))
            )
          )
         (SS (CASE D))
         
         (or 
         	(and (SS (ANSWER $? e n)) 			(satisfied ?sat))
         	(and (not (SS (ANSWER $? e n))) 	(violated  ?sat))
         )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
          then 
	   	 	(?fb addSatisfiedConstraint "sem-4")
		  else
	     	(?fb addMessage "If a noun is in the dative case, and preceded by 'der-' or 'ein-word', the ending on the adjective is -en.")
	   	 	(?fb addViolatedConstraint "sem-4")
	   	 )
)

(defrule sem-5
          (SS (ARTICLE D))
          (not (SS (GENDER P)))
          (or (SS (ANSWER $? e n)) (SS (ANSWER $? e)))
          (or 
          	(SS (CASE N))
           	(and
            	(SS (CASE A))
            	(or (SS (GENDER F)) (SS (GENDER N)))
             )
          )

		 (or         
         	(and (SS (ANSWER $? e)) 		(satisfied ?sat))
         	(and (not (SS (ANSWER $? e)))	(violated  ?sat))
         )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
          then 
		   	 (?fb addSatisfiedConstraint "sem-5")
          else
		     (?fb addMessage "If a noun is singular and preceded by a 'der-word', the adjective ends in -e until the 'der-word' changes.  After this the adjective ends in -en.")
		   	 (?fb addViolatedConstraint "sem-5")
		  )
)
        
(defrule sem-6
          (SS (ARTICLE D))
          (not (SS (GENDER P)))
          (or (SS (ANSWER $? e n)) (SS (ANSWER $? e)))
          (or 
          	(SS (CASE D))
           	(and
            	(SS (CASE A))
            	(SS (GENDER M))
             )
          )
         
         (or
         	(and (SS (ANSWER $? e n)) 		(satisfied ?sat))
         	(and (not (SS (ANSWER $? e n))) (violated ?viol))
         )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
          then 
		   	 (?fb addSatisfiedConstraint "sem-6")
		  else
		     (?fb addMessage "If a noun is singular and preceded by a 'der-word', the adjective ends in -e until the 'der-word' changes.  After this the adjective ends in -en.")
		   	 (?fb addViolatedConstraint "sem-6")
		  )
)
        
        
(defrule sem-7
          (SS (ARTICLE I))
          (SS (ANSWER ? $?))
	
		 (or          
         	(and (or (SS (ANSWER $? e)) (SS (ANSWER $? e n)) (SS (ANSWER $? e r)) (SS (ANSWER $? e s))) 	
         		 (satisfied ?sat))
         	(and (not (or (SS (ANSWER $? e)) (SS (ANSWER $? e n)) (SS (ANSWER $? e r)) (SS (ANSWER $? e s)))) 
         		 (violated ?sat))
		 )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
          then
		   	 (?fb addSatisfiedConstraint "sem-7")
          else
		     (?fb addMessage "If the article is an 'ein-word', the only possible endings are -e, -en, -er and -es.")
		   	 (?fb addViolatedConstraint "sem-7")
		  )
)

        
(defrule sem-8
          (SS (ARTICLE I))
          (SS (CASE D))
          (not (SS (GENDER P)))
          (or (SS (ANSWER $? e)) (SS (ANSWER $? e n)) (SS (ANSWER $? e r)) (SS (ANSWER $? e s)))
         
          (or
          	(and (SS (ANSWER $? e n)) 			(satisfied ?sat))
          	(and (not (SS (ANSWER $? e n))) 	(violated ?sat))
          )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-8")
		 else	
	     	(?fb addMessage "If it follows 'ein-word', before a singular noun, the adjective takes the ending the 'der-word' would have had, until the ein-word changes.  After this the adjective ends in -en.")
	   	 	(?fb addViolatedConstraint "sem-8")
	   	 )
)

(defrule sem-9
          (SS (ARTICLE I))
          (or (SS (CASE N)) (SS (CASE A)))
          (SS (GENDER F))
          (or (SS (ANSWER $? e)) (SS (ANSWER $? e n)) (SS (ANSWER $? e r)) (SS (ANSWER $? e s))) 

		  (or
	         (and (SS (ANSWER $? e)) 		(satisfied ?sat))
	         (and (not (SS (ANSWER $? e))) 	(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-9")
		 else	
	     	(?fb addMessage "If it follows 'ein-word', before a singular noun, the adjective takes the ending the 'der-word' would have had, until the ein-word changes.  After this the adjective ends in -en.")
	   	 	(?fb addViolatedConstraint "sem-9")
	   	 )
)

(defrule sem-10
          (SS (ARTICLE I))
          (or (SS (CASE N)) (SS (CASE A)))
          (SS (GENDER N))
          (or (SS (ANSWER $? e)) (SS (ANSWER $? e n)) (SS (ANSWER $? e r)) (SS (ANSWER $? e s)))

		  (or
	         (and (SS (ANSWER $? e s)) 			(satisfied ?sat))
	         (and (not (SS (ANSWER $? e s))) 	(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-10")
		 else	
	     	(?fb addMessage "If it follows 'ein-word', before a singular noun, the adjective takes the ending the 'der-word' would have had, until the ein-word changes.  After this the adjective ends in -en.")
	   	 	(?fb addViolatedConstraint "sem-10")
	   	 )
)
          
; masculine nominative
(defrule sem-11
          (SS (ARTICLE I))
          (SS (CASE N))
          (SS (GENDER M))
          (or (SS (ANSWER $? e)) (SS (ANSWER $? e n)) (SS (ANSWER $? e r)) (SS (ANSWER $? e s)))

		  (or
	         (and (SS (ANSWER $? e r)) 			(satisfied ?sat))
	         (and (not (SS (ANSWER $? e r))) 	(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-11")
		 else	
	     	(?fb addMessage "If it follows 'ein-word', before a singular noun, the adjective takes the ending the 'der-word' would have had, until the ein-word changes.  After this the adjective ends in -en.")
	   	 	(?fb addViolatedConstraint "sem-11")
	   	 )
)

(defrule sem-12
          (SS (ARTICLE I))
          (SS (CASE A))
          (SS (GENDER M))
          (or (SS (ANSWER $? e)) (SS (ANSWER $? e n)) (SS (ANSWER $? e r)) (SS (ANSWER $? e s)))

		  (or
	         (and (SS (ANSWER $? e n)) 			(satisfied ?sat))
	         (and (not (SS (ANSWER $? e n))) 	(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-12")
		 else	
	     	(?fb addMessage "If it follows 'ein-word', before a singular noun, the adjective takes the ending the 'der-word' would have had, until the ein-word changes.  After this the adjective ends in -en.")
	   	 	(?fb addViolatedConstraint "sem-12")
	   	 )
)
        

(defrule sem-13
          (SS (ARTICLE N))
          (SS (ANSWER ? $?))
          (or (SS (GENDER F)) (SS (GENDER P)))
          (or (SS (CASE N)) (SS (CASE A))) 

		  (or
	         (and (SS (ANSWER $? e)) 			(satisfied ?sat))
	         (and (not (SS (ANSWER $? e)))	 	(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-13")
		 else	
	     	(?fb addMessage "If there is no ein-word or der-word, the adjective takes the ending that the der-word would have if it was present.")
	   	 	(?fb addViolatedConstraint "sem-13")
	   	 )
)

(defrule sem-14
          (SS (ARTICLE N) (GENDER N) (ANSWER ? $?)) 
          (or (SS (CASE N)) (SS (CASE A))) 

		  (or
	         (and (SS (ANSWER $? e s)) 				(satisfied ?sat))
	         (and (not (SS (ANSWER $? e s)))	 	(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-14")
		 else	
	     	(?fb addMessage "If there is no ein-word or der-word, the adjective takes the ending that the der-word would have if it was present.")
	   	 	(?fb addViolatedConstraint "sem-14")
	   	 )
)
         
(defrule sem-15
          (SS (ARTICLE N) (CASE D) (ANSWER ? $?)) 
          (or (SS (GENDER M)) (SS (GENDER N))) 

		  (or
	         (and (SS (ANSWER $? e m)) 				(satisfied ?sat))
	         (and (not (SS (ANSWER $? e m)))	 	(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-15")
		 else	
	     	(?fb addMessage "If there is no ein-word or der-word, the adjective takes the ending that the der-word would have if it was present.")
	   	 	(?fb addViolatedConstraint "sem-15")
	   	 )
)
         
         
(defrule sem-16
          (SS (ARTICLE N) (ANSWER ? $?)) 
          (or 
          	(SS (CASE N) (GENDER M))
          	(SS (CASE D) (GENDER F))
          ) 

		  (or
	         (and (SS (ANSWER $? e r)) 				(satisfied ?sat))
	         (and (not (SS (ANSWER $? e r)))	 	(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-16")
		 else	
	     	(?fb addMessage "If there is no ein-word or der-word, the adjective takes the ending that the der-word would have if it was present.")
	   	 	(?fb addViolatedConstraint "sem-16")
	   	 )
)
         
(defrule sem-17
          (SS (ARTICLE N) (ANSWER ? $?)) 
          (or 
          	(SS (CASE A) (GENDER M))
          	(SS (CASE D) (GENDER P))
          ) 

		  (or
	         (and (SS (ANSWER $? e n)) 				(satisfied ?sat))
	         (and (not (SS (ANSWER $? e n)))	 	(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-17")
		 else	
	     	(?fb addMessage "If there is no ein-word or der-word, the adjective takes the ending that the der-word would have if it was present.")
	   	 	(?fb addViolatedConstraint "sem-17")
	   	 )
)
        
(defrule sem-answer
          (IS (ANSWER ? $?)) 

		  (or
	         (and (SS (ANSWER ? $?)) 				(satisfied ?sat))
	         (and (not (SS (ANSWER ? $?)))	 		(violated ?sat))
		  )
    =>
         (bind ?fb (fetch FEEDBACK))
         (if (eq ?sat SATISFIED)
         then
		   	 (?fb addSatisfiedConstraint "sem-answer")
		 else	
	     	(?fb addMessage "You haven't answered the question!")
	   	 	(?fb addViolatedConstraint "sem-answer")
	   	 )
)
