package jitas.core;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import jess.JessException;
import jitas.exception.JITASException;
import jitas.util.SolutionEvaluator;

/**
 * Compute a Problem object by the JITAS system default algorithm.
 *  
 * @author	DeathMarch (c314g2)
 * @version	08 October 2007
 */
public class DefaultSelectionAlgorithm implements ProblemSelectionAlgorithm{

	/**
	 * Return a Problem object that has been calculated by the default algorithm
	 * @param student The student
	 * @param domain The Domain
	 * @param problems The problems to choose from
	 * @param subdomainName The name of the subdomain
	 * @return The Problem selected by the system
	 */
	public Problem getNextProblem(Student student, Domain domain, SortedMap<Integer, Problem> problems, String subdomainName){
		double difficulty = 0;
		double k1 = 1, k2 = 5, k3 = 10;
		Problem p = null;
		int totalConstraints = 0;
		String domainName = domain.getName();
		
		SortedMap<Double,Problem> problemDifficulty = new TreeMap<Double,Problem>();
		SortedMap<Integer,Problem> unsolvedProblems = new TreeMap<Integer, Problem>(problems);
		
		Set<Integer> solvedProblems = student.getSolvedProblems(domainName, subdomainName);
		
		Iterator<Integer> it = unsolvedProblems.keySet().iterator();
		while (it.hasNext()) {
			Integer ID = it.next();
			if (solvedProblems.contains(ID)) {
				it.remove();
			}
		}

		try {
			for(Map.Entry<Integer,Problem> e : unsolvedProblems.entrySet()){
				difficulty=e.getValue().getDifficulty();
				Feedback feedback = SolutionEvaluator.evaluateSolution(new Solution("ab"), student, domain, subdomainName, false);
				Collection<String> temp = feedback.getViolatedConstraints();
				int size = temp.size();	
				totalConstraints = size;
				if(temp.removeAll(student.getSatisfiedConstraints(domainName))){
					difficulty += k1*(size - temp.size());
					size = temp.size();
				}if(temp.removeAll(student.getViolatedConstraints(domainName))){
					difficulty += k2*(size - temp.size());
					size = temp.size();
				}if(!temp.isEmpty()){
					difficulty += k3*size;
				}
				difficulty = difficulty/(k3 * totalConstraints);
				problemDifficulty.put(difficulty, e.getValue());
			}
			boolean found = false;
			for(Map.Entry<Double,Problem> entry : problemDifficulty.entrySet()){
				if(entry.getKey() > student.getRating(domainName)){
					p = problemDifficulty.get(entry.getKey());
					found = true;
					break;
				}
			}
			if(!found){
				//fall back on default
				System.err.println("falling back on default 1");
				p = student.getNextProblem(domainName, problems);
			}
		} catch (JITASException e) {
			System.err.println("falling back on default 2");
			p = student.getNextProblem(domainName, problems);
		} catch (JessException e) {
			System.err.println("falling back on default 3");
			p = student.getNextProblem(domainName, problems);
		}
		return p;
	}
}
