package jitas.xml.parser;

import jitas.util.Settings;

import java.io.*;
import java.util.HashSet;
import java.util.Set;
import org.xml.sax.*;
import org.w3c.dom.*;

/**
 * This class parses a domains.xml file in order to create a list of domains.
 * 
 * @author	DeathMarch (c314g2)
 * @version	04 July 2007
 */
public class AllDomainsParser extends XMLParser {

	private Document parsedDoc = null;
	
	public AllDomainsParser() throws IOException {
		/* Load the xml file using DOM parser */
		try {
        	parsedDoc = super.loadXML(Settings.getInstance().getFullPathToAllDomains());
        } catch (SAXException se) {
        	System.err.println(se.getMessage());
        }
	}
	
	public AllDomainsParser(String pathName) throws IOException {
		/* Load the xml file using DOM parser */
		try {
        	parsedDoc = super.loadXML(pathName);
        } catch (SAXException se) {
        	System.err.println(se.getMessage());
        }
	}

	/**
	 * Parses the XML file, creating a list of all domain names
	 * @return domainList a List of Domains
	 */
	public Set<String> processTree() {
		Set<String> domainList = new HashSet<String>();
		String name = "";
		
		if(super.isFileEmpty(Settings.getInstance().getFullPathToAllDomains())){
			/* Create a student model*/
	        Element domains = parsedDoc.createElement("domains");
	        /*Append the student model element to the root element*/
			parsedDoc.appendChild(domains);
			super.writeToXML(parsedDoc, Settings.getInstance().getFullPathToAllDomains());
			return null;
		}
		
		NodeList list = parsedDoc.getElementsByTagName("domain");

		/* for each domain */
		for(int i = 0; i < list.getLength(); i++) {
			Node n = list.item(i);
	       
	       	/* for each child (elements eg clause, description) */
			for(Node child = n.getFirstChild(); child != null; child = child.getNextSibling()){     	   
				if(child.getNodeName().equals("name")) {
	       			name = child.getFirstChild().getNodeValue();
	       			break;
	       		}
	       	}
			
	       	/* add the domainName to the domainList */
	       	domainList.add(name);
		}
		return domainList;
	}
}
