package jitas.xml.parser;

import jitas.core.Problem;
import jitas.exception.*;
import jitas.util.MapFromString;

import org.w3c.dom.*;
import org.xml.sax.SAXException;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.*;

/**
 * An XML parser to parse a set of problems from a given file.
 * There will be one such file for each domain.
 * 
 * @author	DeathMarch (c314g2)
 * @version	09 July 2007
 */
public class ProblemParser extends XMLParser{
	
	private Document parsedDoc = null;
	private String pathName = "";
	private int maxDifficulty = -1;
	
	
	/**
	 * The constructor sorts out the arguments, then hands
	 * over the DOM work to the DOM parser; which is used to build 
	 * the DOM tree and then parse the problem docs (.xml).  
	 * @param pathName
	 */	
	public ProblemParser(String pathName)throws ProblemsNotFoundException {
		this.pathName = pathName;
        try {
        	parsedDoc = super.loadXML(pathName);
        } catch (SAXException se) {
        	System.err.println(se.getMessage());
        } catch (FileNotFoundException fnfe) {	
        	throw new ProblemsNotFoundException("ProblemParser.Constructor: Problem file not found");
        } catch (IOException ioe) {
        	System.err.println(ioe);
        }
	}

	/**
	 * Parses the XML file, creating a Problem object for each 
	 * problems represented in the file.
	 * @return problemList a List of problems
	 * @throws DOMException 
	 * @throws InvalidSolutionException 
	 * @throws InvalidSolutionException
	 */
	public SortedMap<Integer, Problem> processTree() throws ProblemsNotFoundException,
	NumberFormatException, DOMException {
		SortedMap<Integer, Problem> problemMap = new TreeMap<Integer, Problem>();
		
		if(super.isFileEmpty(pathName)){
			/* Create a student model*/
	        Element problems = parsedDoc.createElement("problems");
	        /*Append the student model element to the root element*/
			parsedDoc.appendChild(problems);
			super.writeToXML(parsedDoc, pathName);
			return null;
		}
		
		NodeList list = parsedDoc.getElementsByTagName("problem");
		
		/* for each problems */
		   for(int i = 0; i < list.getLength(); i++) {
		       Problem p = new Problem();
		       Node n = list.item(i);
		       /* for each child (elements eg clause, description) */
		       for(Node child = n.getFirstChild(); child != null; child = child.getNextSibling()){     	   
		    	   if(child.getNodeName().equals("id")) {
		    		   p.setId(Integer.parseInt((child.getFirstChild().getNodeValue())));
		           } else if(child.getNodeName().equals("difficulty")) {
		        	   int difficultyLevel = Integer.parseInt((child.getFirstChild().getNodeValue()));
		               p.setDifficulty(difficultyLevel);
		        	   if(difficultyLevel > maxDifficulty) maxDifficulty = difficultyLevel;
		           } else if (child.getNodeName().equals("shortName")) {
		        	   p.setShortName(child.getFirstChild().getNodeValue());
		           } else if(child.getNodeName().equals("problemStatement")) {
		               p.setProblemStatement(child.getFirstChild().getNodeValue());
		           } else if(child.getNodeName().equals("solution")) {
		               p.setSolution(child.getFirstChild().getNodeValue());
		           } else if(child.getNodeName().equals("additionalData")) {
		        	   p.setAdditionalData(MapFromString.convertStringToMap(child.getFirstChild().getNodeValue()));
		           }
		       }
		       /* add the model to the list */
		       problemMap.put((Integer)p.getId(), p);
		   }
		   return problemMap;
	}
	
	
	/**
	 * Gets the highest Problem difficulty which is in the Map 
	 * @return the highest difficulty level of all the Problems
	 */
	public int getMaxDifficulty() {
		return maxDifficulty;
	}
}
