package jitas.unittest;

import java.util.*;
import junit.framework.*;
import jitas.core.*;
import jitas.exception.*;

/* 
 * @author	DeathMarch (c314g2)
 * @version	8 October 2007
 */
public class SubdomainTest extends TestCase {
	
	Subdomain sd;
	Problem prob;
	Problem prob2;

	protected void setUp() {
		sd = new Subdomain("name", "description", "domain"); //had to change Subdomain to public for this
		prob = new Problem();
		prob.setId(1);
		
		try {
			sd.addProblem(prob);
			prob2 = new Problem();
			prob2.setId(2);
			sd.addProblem(prob2);
		} catch (ProblemAlreadyExistsException ex) {
			fail("failed during setup, caught a ProblemAlreadyExistsException ");
		}
		
	}
	
	protected void tearDown() {
		prob = null;
		sd = null;
		prob2 = null;
		
	}
	

	
	public void testGetName() {
		assertTrue(sd.getName().equals("name"));
	}


	public void testGetDescription() {
		assertTrue(sd.getDescription().equals("description"));
	}
	

	public void  testGetProblems() {
		int numAsserts = 0;
	
		try {
			SortedMap<Integer, Problem> probMap = sd.getProblems();
	
			assertTrue(probMap.get(probMap.firstKey()).getId() == 1);
			numAsserts = 1;
			assertTrue(probMap.get(probMap.lastKey()).getId() == 2);
			numAsserts = 2;
			/*The following should throw an exception */
			assertFalse(probMap.get(probMap.lastKey()).getId() == -1);
			
			//fail("Exception was expected but wasn't thrown");
		} 
		catch (NoSuchElementException ex) {
			if(numAsserts <2)
				fail("SubdomainTest testGetProblems caught an unexpected exception\n" + ex.toString());
			/* otherwise the Exception was expected */
		}
		
		catch (Exception ex) {
			System.out.println(ex.toString());
			fail("caught exception in testGetProblems()");
		}
	
	}
	
	
	/**
	 * 
	 */
	public void testAddProblem() {
		try {
			assertFalse(sd.addProblem(null));
			
		} catch (ProblemAlreadyExistsException ex) {
			fail("Caught ProblemAlreadyExistsException in SubdomainTest: testAddProblem");
		}
		
		try {
			sd.addProblem(prob2);
			fail("expected an exception, and one wasn't thrown in testAddProblem");
			
		} catch (ProblemAlreadyExistsException ex) {
			return;
		}
	}
	
	/*
	 * Returns the list of constraints associated with this subdomain (if there are any)
	 * @return The list of constraints, or null if there are none
	 */
	/*public void  testGetConstraints() {
			fail("not yet implemented");
	}*/
	
	/*
	 * Load problems for the domain from the correct xml file
	 * The file is "domains/<domain-name>/problems.xml"
	 */
	/*public void testLoadProblems() throws ProblemsNotFoundException {
		fail("not yet implemented");
	}
	
	public void testLoadCustomConstraints() throws ConstraintNotFoundException{
		fail("not yet implemented");
	}*/
	
	/*
	 * Standard equals override, based on the Subdomain name
	 * @return true if the objects are equal
	 */
	/*public void  testEquals(Object object) {
		fail("not yet implemented");
	}*/
	
	/*
	 * Standard compareTo override, based on the Subdomain name
	 * @param object the Object to be compared
	 * @return a negative integer, zero, or a positive integer as this object
	 * is less than, equal to, or greater than the specified object.
	 * @throws ClassCastException
	 */
	/*public void  testCompareTo(Object object) throws ClassCastException {
		fail("not yet implemented");
	}*/
	

	/*
	 * Standard toString override
	 */
	/*public void  testToString() {
		fail("not yet implemented");
	}*/
}
